@@ -16,14 +16,14 @@
 
 import sys
 import os
-import gtk
+from gi.repository import Gtk
 import logging
-import webbrowser
 
-from pithos.pithosconfig import getdatapath
-from pithos import SearchDialog
+from .pithosconfig import get_ui_file
+from .util import open_browser
+from . import SearchDialog
 
-class StationsDialog(gtk.Dialog):
+class StationsDialog(Gtk.Dialog):
     __gtype_name__ = "StationsDialog"
 
     def __init__(self):
@@ -55,35 +55,35 @@ class StationsDialog(gtk.Dialog):
         self.searchDialog = None
         
         self.modelfilter = self.model.filter_new()
-        self.modelfilter.set_visible_func(lambda m, i: m.get_value(i, 0) and not  m.get_value(i, 0).isQuickMix)
+        self.modelfilter.set_visible_func(lambda m, i, d: m.get_value(i, 0) and not  m.get_value(i, 0).isQuickMix)
 
-        self.modelsortable = gtk.TreeModelSort(self.modelfilter)
+        self.modelsortable = Gtk.TreeModelSort.sort_new_with_model(self.modelfilter)
         """
         @todo Leaving it as sorting by date added by default. 
         Probably should make a radio select in the window or an option in program options for user preference
         """
-#        self.modelsortable.set_sort_column_id(1, gtk.SORT_ASCENDING)
+#        self.modelsortable.set_sort_column_id(1, Gtk.SortType.ASCENDING)
         
         self.treeview = self.builder.get_object("treeview")
         self.treeview.set_model(self.modelsortable)
         self.treeview.connect('button_press_event', self.on_treeview_button_press_event)
         
-        name_col   = gtk.TreeViewColumn()
+        name_col   = Gtk.TreeViewColumn()
         name_col.set_title("Name")
-        render_text = gtk.CellRendererText()
+        render_text = Gtk.CellRendererText()
         render_text.set_property('editable', True)
         render_text.connect("edited", self.station_renamed)
-        name_col.pack_start(render_text, expand=True)
+        name_col.pack_start(render_text, True)
         name_col.add_attribute(render_text, "text", 1)
         name_col.set_expand(True)
         name_col.set_sort_column_id(1)
         self.treeview.append_column(name_col)
         
-        qm_col   = gtk.TreeViewColumn()
+        qm_col   = Gtk.TreeViewColumn()
         qm_col.set_title("In QuickMix")
-        render_toggle = gtk.CellRendererToggle()
-        qm_col.pack_start(render_toggle, expand=True)
-        def qm_datafunc(column, cell, model, iter):
+        render_toggle = Gtk.CellRendererToggle()
+        qm_col.pack_start(render_toggle, True)
+        def qm_datafunc(column, cell, model, iter, data=None):
             if model.get_value(iter,0).useQuickMix:
                 cell.set_active(True)
             else:
@@ -102,7 +102,7 @@ class StationsDialog(gtk.Dialog):
     def station_renamed(self, cellrenderertext, path, new_text):
         station = self.modelfilter[path][0]
         self.worker_run(station.rename, (new_text,), context='net', message="Renaming Station...")
-        self.model[self.modelfilter.convert_path_to_child_path(path)][1] = new_text
+        self.model[self.modelfilter.convert_path_to_child_path(Gtk.TreePath(path))][1] = new_text
         
     def selected_station(self):
         sel = self.treeview.get_selection().get_selected()
@@ -119,7 +119,7 @@ class StationsDialog(gtk.Dialog):
                 path, col, cellx, celly = pthinfo
                 treeview.grab_focus()
                 treeview.set_cursor( path, col, 0)
-                self.station_menu.popup( None, None, None, event.button, time)
+                self.station_menu.popup(None, None, None, None, event.button, time)
             return True
             
     def on_menuitem_listen(self, widget):
@@ -128,7 +128,7 @@ class StationsDialog(gtk.Dialog):
         self.hide()
         
     def on_menuitem_info(self, widget):
-        webbrowser.open(self.selected_station().info_url)
+        open_browser(self.selected_station().info_url)
         
     def on_menuitem_rename(self, widget):
         sel = self.treeview.get_selection().get_selected()
@@ -154,6 +154,7 @@ class StationsDialog(gtk.Dialog):
             self.searchDialog.present()
         else:
             self.searchDialog = SearchDialog.NewSearchDialog(self.worker_run)
+            self.searchDialog.set_transient_for(self)
             self.searchDialog.show_all()
             self.searchDialog.connect("response", self.add_station_cb)
             
@@ -161,7 +162,7 @@ class StationsDialog(gtk.Dialog):
         self.pithos.refresh_stations(self.pithos)
         
     def add_station_cb(self, dialog, response):
-        print "in add_station_cb", dialog.result, response
+        logging.info("in add_station_cb {} {}".format(dialog.result, response))
         if response == 1:
             self.worker_run("add_station_by_music_id", (dialog.result.musicId,), self.station_added, "Creating station...")
         dialog.hide()
@@ -201,13 +202,8 @@ def NewStationsDialog(pithos):
     
     """
 
-    #look for the ui file that describes the ui
-    ui_filename = os.path.join(getdatapath(), 'ui', 'StationsDialog.ui')
-    if not os.path.exists(ui_filename):
-        ui_filename = None
-
-    builder = gtk.Builder()
-    builder.add_from_file(ui_filename)    
+    builder = Gtk.Builder()
+    builder.add_from_file(get_ui_file('stations'))    
     dialog = builder.get_object("stations_dialog")
     dialog.finish_initializing(builder, pithos)
     return dialog
@@ -215,5 +211,5 @@ def NewStationsDialog(pithos):
 if __name__ == "__main__":
     dialog = NewStationsDialog()
     dialog.show()
-    gtk.main()
+    Gtk.main()
 
