@@ -2,39 +2,35 @@
 ### BEGIN LICENSE
 # Copyright (C) 2010 Kevin Mehall <km@kevinmehall.net>
 # Copyright (C) 2012 Christopher Eby <kreed@kreed.org>
-#This program is free software: you can redistribute it and/or modify it 
-#under the terms of the GNU General Public License version 3, as published 
+#This program is free software: you can redistribute it and/or modify it
+#under the terms of the GNU General Public License version 3, as published
 #by the Free Software Foundation.
 #
-#This program is distributed in the hope that it will be useful, but 
-#WITHOUT ANY WARRANTY; without even the implied warranties of 
-#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
+#This program is distributed in the hope that it will be useful, but
+#WITHOUT ANY WARRANTY; without even the implied warranties of
+#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 #PURPOSE.  See the GNU General Public License for more details.
 #
-#You should have received a copy of the GNU General Public License along 
+#You should have received a copy of the GNU General Public License along
 #with this program.  If not, see <http://www.gnu.org/licenses/>.
 ### END LICENSE
 
-from pithos.pandora.blowfish import Blowfish
-from pithos.pandora import pandora_keys
+"""Pandora JSON v5 API
+
+See http://6xq.net/playground/pandora-apidoc/json/ for API documentation.
+"""
+
+from .blowfish import Blowfish
+# from Crypto.Cipher import Blowfish
+from xml.dom import minidom
+import re
 import json
 import logging
 import time
-import urllib
-import urllib2
-
-# This is an implementation of the Pandora JSON API using Android partner
-# credentials.
-# See http://pan-do-ra-api.wikia.com/wiki/Json/5 for API documentation.
-
-PROTOCOL_VERSION = '5'
-RPC_URL = "://tuner.pandora.com/services/json/?"
-DEVICE_MODEL = 'android-generic'
-PARTNER_USERNAME = 'android'
-PARTNER_PASSWORD = 'AC7IBG09A3DTSYM4R41UJWL07VLN8JI7'
+import urllib.request, urllib.parse, urllib.error
+import codecs
 
 HTTP_TIMEOUT = 30
-AUDIO_FORMAT = 'aacplus'
 USER_AGENT = 'pithos'
 
 RATE_BAN = 'ban'
@@ -42,13 +38,19 @@ RATE_LOVE = 'love'
 RATE_NONE = None
 
 API_ERROR_API_VERSION_NOT_SUPPORTED = 11
+API_ERROR_COUNTRY_NOT_SUPPORTED = 12
 API_ERROR_INSUFFICIENT_CONNECTIVITY = 13
 API_ERROR_READ_ONLY_MODE = 1000
 API_ERROR_INVALID_AUTH_TOKEN = 1001
 API_ERROR_INVALID_LOGIN = 1002
+API_ERROR_LISTENER_NOT_AUTHORIZED = 1003
+API_ERROR_PARTNER_NOT_AUTHORIZED = 1010
+API_ERROR_PLAYLIST_EXCEEDED = 1039
 
 PLAYLIST_VALIDITY_TIME = 60*60*3
 
+NAME_COMPARE_REGEX = re.compile(r'[^A-Za-z0-9]')
+
 class PandoraError(IOError):
     def __init__(self, message, status=None, submsg=None):
         self.status = status
@@ -60,25 +62,30 @@ class PandoraNetError(PandoraError): pass
 class PandoraAPIVersionError(PandoraError): pass
 class PandoraTimeout(PandoraNetError): pass
 
-
-blowfish_encode = Blowfish(pandora_keys.out_key_p, pandora_keys.out_key_s)
-
 def pad(s, l):
-    return s + "\0" * (l - len(s))
-
-def pandora_encrypt(s):
-    return "".join([blowfish_encode.encrypt(pad(s[i:i+8], 8)).encode('hex') for i in xrange(0, len(s), 8)])
-
-blowfish_decode = Blowfish(pandora_keys.in_key_p, pandora_keys.in_key_s)
-
-def pandora_decrypt(s):
-    return "".join([blowfish_decode.decrypt(pad(s[i:i+16].decode('hex'), 8)) for i in xrange(0, len(s), 16)]).rstrip('\x08')
-
+    return s + b'\0' * (l - len(s))
 
 class Pandora(object):
+    """Access the Pandora API
+
+    To use the Pandora class, make sure to call :py:meth:`set_audio_quality`
+    and :py:meth:`connect` methods.
+
+    Get information from Pandora using:
+
+    - :py:meth:`get_stations` which populates the :py:attr:`stations` attribute
+    - :py:meth:`search` to find songs to add to stations or create a new station with
+    - :py:meth:`json_call` call into the JSON API directly
+    """
     def __init__(self):
-        self.set_proxy(None)
-        self.set_audio_format(AUDIO_FORMAT)
+        self.opener = urllib.request.build_opener()
+        pass
+
+    def pandora_encrypt(self, s):
+        return b''.join([codecs.encode(self.blowfish_encode.encrypt(pad(s[i:i+8], 8)), 'hex_codec') for i in range(0, len(s), 8)])
+
+    def pandora_decrypt(self, s):
+        return b''.join([self.blowfish_decode.decrypt(pad(codecs.decode(s[i:i+16], 'hex_codec'), 8)) for i in range(0, len(s), 16)]).rstrip(b'\x08')
 
     def json_call(self, method, args={}, https=False, blowfish=True):
         url_arg_strings = []
@@ -87,13 +94,13 @@ class Pandora(object):
         if self.userId:
             url_arg_strings.append('user_id=%s'%self.userId)
         if self.userAuthToken:
-            url_arg_strings.append('auth_token=%s'%urllib.quote_plus(self.userAuthToken))
+            url_arg_strings.append('auth_token=%s'%urllib.parse.quote_plus(self.userAuthToken))
         elif self.partnerAuthToken:
-            url_arg_strings.append('auth_token=%s'%urllib.quote_plus(self.partnerAuthToken))
+            url_arg_strings.append('auth_token=%s'%urllib.parse.quote_plus(self.partnerAuthToken))
 
         url_arg_strings.append('method=%s'%method)
         protocol = 'https' if https else 'http'
-        url = protocol + RPC_URL + '&'.join(url_arg_strings)
+        url = protocol + self.rpcUrl + '&'.join(url_arg_strings)
 
         if self.time_offset:
             args['syncTime'] = int(time.time()+self.time_offset)
@@ -101,27 +108,27 @@ class Pandora(object):
             args['userAuthToken'] = self.userAuthToken
         elif self.partnerAuthToken:
             args['partnerAuthToken'] = self.partnerAuthToken
-        data = json.dumps(args)
+        data = json.dumps(args).encode('utf-8')
 
         logging.debug(url)
         logging.debug(data)
 
         if blowfish:
-            data = pandora_encrypt(data)
+            data = self.pandora_encrypt(data)
 
         try:
-            req = urllib2.Request(url, data, {'User-agent': USER_AGENT, 'Content-type': 'text/plain'})
+            req = urllib.request.Request(url, data, {'User-agent': USER_AGENT, 'Content-type': 'text/plain'})
             response = self.opener.open(req, timeout=HTTP_TIMEOUT)
-            text = response.read()
-        except urllib2.HTTPError as e:
+            text = response.read().decode('utf-8')
+        except urllib.error.HTTPError as e:
             logging.error("HTTP error: %s", e)
             raise PandoraNetError(str(e))
-        except urllib2.URLError as e:
+        except urllib.error.URLError as e:
             logging.error("Network error: %s", e)
-            if e.reason[0] == 'timed out':
+            if e.reason.strerror == 'timed out':
                 raise PandoraTimeout("Network error", submsg="Timeout")
             else:
-                raise PandoraNetError("Network error", submsg=e.reason[1])
+                raise PandoraNetError("Network error", submsg=e.reason.strerror)
 
         logging.debug(text)
 
@@ -134,6 +141,9 @@ class Pandora(object):
 
             if code == API_ERROR_INVALID_AUTH_TOKEN:
                 raise PandoraAuthTokenInvalid(msg)
+            elif code == API_ERROR_COUNTRY_NOT_SUPPORTED:
+                 raise PandoraError("Pandora not available", code,
+                    submsg="Pandora is not available in your country.")
             elif code == API_ERROR_API_VERSION_NOT_SUPPORTED:
                 raise PandoraAPIVersionError(msg)
             elif code == API_ERROR_INSUFFICIENT_CONNECTIVITY:
@@ -144,30 +154,58 @@ class Pandora(object):
                     submsg="Pandora is in read-only mode as it is performing maintenance. Try again later.")
             elif code == API_ERROR_INVALID_LOGIN:
                 raise PandoraError("Login Error", code, submsg="Invalid username or password")
+            elif code == API_ERROR_LISTENER_NOT_AUTHORIZED:
+                raise PandoraError("Pandora Error", code,
+                    submsg="A Pandora One account is required to access this feature. Uncheck 'Pandora One' in Settings.")
+            elif code == API_ERROR_PARTNER_NOT_AUTHORIZED:
+                raise PandoraError("Login Error", code,
+                    submsg="Invalid Pandora partner keys. A Pithos update may be required.")
+            elif code == API_ERROR_PLAYLIST_EXCEEDED:
+                raise PandoraError("Playlist Error", code,
+                    submsg="You have requested too many playlists. Try again later.")
             else:
                 raise PandoraError("Pandora returned an error", code, "%s (code %d)"%(msg, code))
 
         if 'result' in tree:
             return tree['result']
 
-    def set_audio_format(self, fmt):
-        self.audio_format = ['aacplus', 'mp3', 'mp3-hifi'].index(fmt)
+    def set_audio_quality(self, fmt):
+        """Set the desired audio quality
 
-    def set_proxy(self, proxy):
-        if proxy:
-            proxy_handler = urllib2.ProxyHandler({'http': proxy})
-            self.opener = urllib2.build_opener(proxy_handler)  
-        else:
-            self.opener = urllib2.build_opener()     
+        Used by the :py:attr:`Song.audioUrl` property.
 
-    def connect(self, user, password):
-        self.partnerId = self.userId = self.partnerAuthToken = self.userAuthToken = self.time_offset = None
+        :param fmt: An audio quality format from :py:data:`pithos.pandora.data.valid_audio_formats`
+        """
+        self.audio_quality = fmt
+
+    def set_url_opener(self, opener):
+        self.opener = opener
+
+    def connect(self, client, user, password):
+        """Connect to the Pandora API and log the user in
+
+        :param client:   The client ID from :py:data:`pithos.pandora.data.client_keys`
+        :param user:     The user's login email
+        :param password: The user's login password
+        """
+        self.partnerId = self.userId = self.partnerAuthToken = None
+        self.userAuthToken = self.time_offset = None
+
+        self.rpcUrl = client['rpcUrl']
+        self.blowfish_encode = Blowfish(client['encryptKey'].encode('utf-8'))
+        self.blowfish_decode = Blowfish(client['decryptKey'].encode('utf-8'))
+
+        partner = self.json_call('auth.partnerLogin', {
+            'deviceModel': client['deviceModel'],
+            'username': client['username'], # partner username
+            'password': client['password'], # partner password
+            'version': client['version']
+            },https=True, blowfish=False)
 
-        partner = self.json_call('auth.partnerLogin', {'deviceModel': DEVICE_MODEL, 'username': PARTNER_USERNAME, 'password': PARTNER_PASSWORD, 'version': PROTOCOL_VERSION}, https=True, blowfish=False)
         self.partnerId = partner['partnerId']
         self.partnerAuthToken = partner['partnerAuthToken']
 
-        pandora_time = int(pandora_decrypt(partner['syncTime'])[4:14])
+        pandora_time = int(self.pandora_decrypt(partner['syncTime'].encode('utf-8'))[4:14])
         self.time_offset = pandora_time - time.time()
         logging.info("Time offset is %s", self.time_offset)
 
@@ -187,6 +225,8 @@ class Pandora(object):
                 if i.id in self.quickMixStationIds:
                     i.useQuickMix = True
 
+        return self.stations
+
     def save_quick_mix(self):
         stationIds = []
         for i in self.stations:
@@ -245,7 +285,7 @@ class Station(object):
 
     def get_playlist(self):
         logging.info("pandora: Get Playlist")
-        playlist = self.pandora.json_call('station.getPlaylist', {'stationToken': self.idToken, 'additionalAudioUrl': 'HTTP_64_AACPLUS_ADTS,HTTP_128_MP3,HTTP_192_MP3'}, https=True)
+        playlist = self.pandora.json_call('station.getPlaylist', {'stationToken': self.idToken}, https=True)
         songs = []
         for i in playlist['items']:
             if 'songName' in i: # check for ads
@@ -267,33 +307,87 @@ class Station(object):
         logging.info("pandora: Deleting Station")
         self.pandora.json_call('station.deleteStation', {'stationToken': self.idToken})
 
+    def __repr__(self):
+        return '<{}.{} {} "{}">'.format(
+            __name__,
+            __class__.__name__,
+            self.id,
+            self.name,
+        )
+
 class Song(object):
     def __init__(self, pandora, d):
         self.pandora = pandora
 
         self.album = d['albumName']
         self.artist = d['artistName']
-        self.audioUrl = d['additionalAudioUrl'][self.pandora.audio_format]
-        self.fileGain = d['trackGain']
+        self.audioUrlMap = d['audioUrlMap']
         self.trackToken = d['trackToken']
         self.rating = RATE_LOVE if d['songRating'] == 1 else RATE_NONE # banned songs won't play, so we don't care about them
         self.stationId = d['stationId']
-        self.title = d['songName']
+        self.songName = d['songName']
         self.songDetailURL = d['songDetailUrl']
-        self.albumDetailURL = d['albumDetailUrl']
+        self.songExplorerUrl = d['songExplorerUrl']
         self.artRadio = d['albumArtUrl']
 
+        self.bitrate = None
+        self.is_ad = None  # None = we haven't checked, otherwise True/False
         self.tired=False
         self.message=''
+        self.duration = None
+        self.position = None
         self.start_time = None
         self.finished = False
         self.playlist_time = time.time()
         self.feedbackId = None
 
+    @property
+    def title(self):
+        if not hasattr(self, '_title'):
+            # the actual name of the track, minus any special characters (except dashes) is stored
+            # as the last part of the songExplorerUrl, before the args.
+            explorer_name = self.songExplorerUrl.split('?')[0].split('/')[-1]
+            clean_expl_name = NAME_COMPARE_REGEX.sub('', explorer_name).lower()
+            clean_name = NAME_COMPARE_REGEX.sub('', self.songName).lower()
+
+            if clean_name == clean_expl_name:
+                self._title = self.songName
+            else:
+                try:
+                    xml_data = urllib.urlopen(self.songExplorerUrl)
+                    dom = minidom.parseString(xml_data.read())
+                    attr_value = dom.getElementsByTagName('songExplorer')[0].attributes['songTitle'].value
+
+                    # Pandora stores their titles for film scores and the like as 'Score name: song name'
+                    self._title = attr_value.replace('{0}: '.format(self.songName), '', 1)
+                except:
+                    self._title = self.songName
+        return self._title
+
+    @property
+    def audioUrl(self):
+        quality = self.pandora.audio_quality
+        try:
+            q = self.audioUrlMap[quality]
+            logging.info("Using audio quality %s: %s %s", quality, q['bitrate'], q['encoding'])
+            return q['audioUrl']
+        except KeyError:
+            logging.warn("Unable to use audio format %s. Using %s",
+                           quality, list(self.audioUrlMap.keys())[0])
+            return list(self.audioUrlMap.values())[0]['audioUrl']
+
     @property
     def station(self):
         return self.pandora.get_station_by_id(self.stationId)
 
+    def get_duration_sec (self):
+      if self.duration is not None:
+        return self.duration / 1000000000
+
+    def get_position_sec (self):
+      if self.position is not None:
+        return self.position / 1000000000
+
     def rate(self, rating):
         if self.rating != rating:
             self.station.transformIfShared()
@@ -328,6 +422,17 @@ class Song(object):
     def is_still_valid(self):
         return (time.time() - self.playlist_time) < PLAYLIST_VALIDITY_TIME
 
+    def __repr__(self):
+        return '<{}.{} {} "{}" by "{}" from "{}">'.format(
+            __name__,
+            __class__.__name__,
+            self.trackToken,
+            self.songName,
+            self.artist,
+            self.album,
+        )
+
+
 class SearchResult(object):
     def __init__(self, resultType, d):
         self.resultType = resultType
