@@ -1,37 +1,59 @@
 # -*- coding: utf-8; tab-width: 4; indent-tabs-mode: nil; -*-
 ### BEGIN LICENSE
 # Copyright (C) 2010-2012 Kevin Mehall <km@kevinmehall.net>
-#This program is free software: you can redistribute it and/or modify it 
-#under the terms of the GNU General Public License version 3, as published 
+#This program is free software: you can redistribute it and/or modify it
+#under the terms of the GNU General Public License version 3, as published
 #by the Free Software Foundation.
 #
-#This program is distributed in the hope that it will be useful, but 
-#WITHOUT ANY WARRANTY; without even the implied warranties of 
-#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
+#This program is distributed in the hope that it will be useful, but
+#WITHOUT ANY WARRANTY; without even the implied warranties of
+#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 #PURPOSE.  See the GNU General Public License for more details.
 #
-#You should have received a copy of the GNU General Public License along 
+#You should have received a copy of the GNU General Public License along
 #with this program.  If not, see <http://www.gnu.org/licenses/>.
 ### END LICENSE
 
 # where your project will head for your data (for instance, images and ui files)
-# by default, this is ../data, relative your trunk layout
-__pithos_data_directory__ = '../data/'
+# by default, this is data, relative your trunk layout
+__pithos_data_directory__ = 'data/'
 __license__ = 'GPL-3'
 
-VERSION = '0.3.17'
-
+VERSION = '1.1.1'
 
 import os
 
 class project_path_not_found(Exception):
     pass
 
-valid_audio_formats = [
-    'aacplus',
-    'mp3',
-    'mp3-hifi',
-]
+ui_files = {
+    'about': 'AboutPithosDialog.ui',
+    'preferences': 'PreferencesPithosDialog.ui',
+    'search': 'SearchDialog.ui',
+    'stations': 'StationsDialog.ui',
+    'main': 'PithosWindow.ui',
+    'menu': 'app_menu.ui'
+}
+
+media_files = {
+    'icon': 'icon.svg',
+    'rate': 'rate_bg.png',
+    'album': 'album_default.png'
+}
+
+def get_media_file(name):
+    media = os.path.join(getdatapath(), 'media', media_files[name])
+    if not os.path.exists(media):
+        media = None
+        
+    return media
+
+def get_ui_file(name):
+    ui_filename = os.path.join(getdatapath(), 'ui', ui_files[name])
+    if not os.path.exists(ui_filename):
+        ui_filename = None
+        
+    return ui_filename
 
 def get_data_file(*path_segments):
     """Get the full path to a data file.
@@ -63,4 +85,4 @@ def getdatapath():
         raise project_path_not_found
 
 if __name__=='__main__':
-    print VERSION
+    print(VERSION)
