@@ -14,53 +14,72 @@
 #with this program.  If not, see <http://www.gnu.org/licenses/>.
 ### END LICENSE
 
-import gtk
+import os
+from gi.repository import Gtk
 from pithos.pithosconfig import get_data_file
 from pithos.plugin import PithosPlugin
 
-# Check if appindicator is available on the system
+# Use appindicator if on Unity and installed
 try:
-    import appindicator
-    indicator_capable = True
+    if os.environ['XDG_CURRENT_DESKTOP'] == 'Unity':
+        from gi.repository import AppIndicator3 as AppIndicator
+        indicator_capable = True
+    else:
+        indicator_capable = False
 except:
     indicator_capable = False
 
 class PithosNotificationIcon(PithosPlugin):    
     preference = 'show_icon'
+    description = 'Adds pithos icon to system tray'
             
     def on_prepare(self):
         if indicator_capable:
-            self.ind = appindicator.Indicator("pithos", \
-                                  "pithos-mono", \
-                                   appindicator.CATEGORY_APPLICATION_STATUS, \
+            self.ind = AppIndicator.Indicator.new_with_path("pithos-tray-icon", \
+                                  "pithos-tray-icon", \
+                                   AppIndicator.IndicatorCategory.APPLICATION_STATUS, \
                                    get_data_file('media'))
     
     def on_enable(self):
-        self.visible = True
         self.delete_callback_handle = self.window.connect("delete-event", self.toggle_visible)
         self.state_callback_handle = self.window.connect("play-state-changed", self.play_state_changed)
         self.song_callback_handle = self.window.connect("song-changed", self.song_changed)
         
         if indicator_capable:
-            self.ind.set_status(appindicator.STATUS_ACTIVE)      
+            self.ind.set_status(AppIndicator.IndicatorStatus.ACTIVE)
         else:
-            self.statusicon = gtk.status_icon_new_from_file(get_data_file('media', 'icon.png'))
+            icon_info = Gtk.IconTheme.lookup_icon (Gtk.IconTheme.get_default(), 'pithos-tray-icon', 48, 0)
+            if icon_info and Gtk.IconInfo.get_filename (icon_info):
+                filename = Gtk.IconInfo.get_filename (icon_info)
+            else:
+                filename = get_data_file('media', 'pithos-tray-icon.png')
+
+            self.statusicon = Gtk.StatusIcon.new ()
+            self.statusicon.set_from_file (filename)
             self.statusicon.connect('activate', self.toggle_visible)
         
         self.build_context_menu()
-       
+
+    def scroll(self, steps):
+        if indicator_capable:
+            direction = steps.value_nick
+        else:
+            direction = steps.direction.value_nick
+
+        if direction == 'down':
+            self.window.adjust_volume(-1)
+        elif direction == 'up':
+            self.window.adjust_volume(+1)
+
     def build_context_menu(self):
-        menu = gtk.Menu()
+        menu = Gtk.Menu()
         
-        def button(text, action, icon=None):
-            if icon == 'check':
-                item = gtk.CheckMenuItem(text)
+        def button(text, action, checked=False):
+            if checked:
+                item = Gtk.CheckMenuItem(text)
                 item.set_active(True)
-            elif icon:
-                item = gtk.ImageMenuItem(text)
-                item.set_image(gtk.image_new_from_stock(icon, gtk.ICON_SIZE_MENU))
             else:
-                item = gtk.MenuItem(text)
+                item = Gtk.MenuItem(text)
             item.connect('activate', action) 
             item.show()
             menu.append(item)
@@ -68,21 +87,24 @@ class PithosNotificationIcon(PithosPlugin):
         
         if indicator_capable:
             # We have to add another entry for show / hide Pithos window
-            self.visible_check = button("Show Pithos", self._toggle_visible, 'check')
+            self.visible_check = button("Show Pithos", self._toggle_visible, True)
         
-        self.playpausebtn = button("Pause", self.window.playpause, gtk.STOCK_MEDIA_PAUSE)
-        button("Skip",  self.window.next_song,                     gtk.STOCK_MEDIA_NEXT)
-        button("Love",  (lambda *i: self.window.love_song()),      gtk.STOCK_ABOUT)
-        button("Ban",   (lambda *i: self.window.ban_song()),       gtk.STOCK_CANCEL)
-        button("Tired", (lambda *i: self.window.tired_song()),     gtk.STOCK_JUMP_TO)
-        button("Quit",  self.window.quit,                          gtk.STOCK_QUIT )
+        self.playpausebtn = button("Pause", self.window.playpause)
+        button("Skip",  self.window.next_song)
+        button("Love",  (lambda *i: self.window.love_song()))
+        button("Ban",   (lambda *i: self.window.ban_song()))
+        button("Tired", (lambda *i: self.window.tired_song()))
+        button("Quit",  self.window.quit)
 
         # connect our new menu to the statusicon or the appindicator
         if indicator_capable:
             self.ind.set_menu(menu)
+            # Disabled because of https://bugs.launchpad.net/variety/+bug/1071598
+            #self.ind.connect('scroll-event', lambda _x, _y, steps: self.scroll(steps))
         else:
             self.statusicon.connect('popup-menu', self.context_menu, menu)
-            
+            self.statusicon.connect('scroll-event', lambda _, steps: self.scroll(steps))
+
         self.menu = menu
 
 
@@ -92,30 +114,25 @@ class PithosNotificationIcon(PithosPlugin):
         button = self.playpausebtn
         if not playing:
             button.set_label("Play")
-            button.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_MENU))
-
         else:
             button.set_label("Pause")
-            button.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_MENU))
             
         if indicator_capable: # menu needs to be reset to get updated icon
             self.ind.set_menu(self.menu)
 
     def song_changed(self, window, song):
         if not indicator_capable:
-            self.statusicon.set_tooltip("%s by %s"%(song.title, song.artist))
+            self.statusicon.set_tooltip_text("%s by %s"%(song.title, song.artist))
         
     def _toggle_visible(self, *args):
-        if self.visible:
-            self.window.hide()
-        else:
+        self.window.set_visible(not self.window.get_visible())
+
+        if self.window.get_visible(): # Ensure it's on top
             self.window.bring_to_top()
         
-        self.visible = not self.visible
-        
     def toggle_visible(self, *args):
         if hasattr(self, 'visible_check'):
-            self.visible_check.set_active(not self.visible)
+            self.visible_check.set_active(not self.window.get_visible())
         else:
             self._toggle_visible()
         
@@ -125,11 +142,11 @@ class PithosNotificationIcon(PithosPlugin):
        if button == 3: 
            if data: 
                data.show_all() 
-               data.popup(None, None, None, 3, time)
+               data.popup(None, None, None, None, 3, time)
     
     def on_disable(self):
         if indicator_capable:
-            self.ind.set_status(appindicator.STATUS_PASSIVE)
+            self.ind.set_status(AppIndicator.IndicatorStatus.PASSIVE)
         else:
             self.statusicon.set_visible(False)
             
